locals {
  templatevars = {
    public_key = var.guest_ssh_key_public,
    ssh_username = var.guest_ssh_user
  }
}

data "template_cloudinit_config" "example" {
  gzip = true
  base64_encode = true
  part {
    content_type = "text/cloud-config"
    content = <<-EOF
      #cloud-config
      users:
        - name: ${local.templatevars.ssh_username}
          lock_passwd: true
          sudo: ['ALL=(ALL) NOPASSWD:ALL']
          groups: sudo
          shell: /bin/bash
          ssh-authorized-keys:
            - ${local.templatevars.public_key}
      EOF
  }
}

resource "vsphere_virtual_machine" "vm" {
    count = var.quantity

    name = "${var.cluster_name}-${var.name_prefix}-${count.index}"
    resource_pool_id = var.resource_pool_id
    datastore_cluster_id = var.datastore_cluster_id

    folder = var.folder

    guest_id = var.template_vm.guest_id

    network_interface {
        network_id = var.network_id
    }
    num_cpus = var.num_cpu
    memory   = var.ram
    disk {
        label            = "${var.cluster_name}-${var.name_prefix}-${count.index}"
        size             = var.disk_size
        thin_provisioned = var.template_vm.disks.0.thin_provisioned
    }

    extra_config = {
      "guestinfo.userdata"          = data.template_cloudinit_config.example.rendered
      "guestinfo.userdata.encoding" = "gzip+base64"
      "guestinfo.metadata"          = <<-EOT
         { "local-hostname": "${var.cluster_name}-${var.name_prefix}-${count.index}" }
      EOT
    }
    clone {
        template_uuid = var.template_vm.id

        customize {
            network_interface{}
            linux_options {
                host_name = "${var.cluster_name}-${var.name_prefix}-${count.index}"
                domain    = "${var.cluster_name}.internal"
            }
        }
    }
}
