#    Copyright 2020 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import pytest

from si_tests import logger
from si_tests import settings
from si_tests.managers import bootstrap_manager

LOG = logger.logger


class TestProvisionOsRegionalClusters():

    @pytest.mark.usefixtures("introspect_openstack_seed_regional_deploy_objects")
    def test_deploy_os_regional(self, kaas_manager,
                                openstack_client, check_sg, show_step):
        """Deploy Regional cluster on KaaS mgmt

        Scenario:
            1. Pass Region var and run bootsrap.sh deploy_regional
            2. Check result of deployment
        """

        seed_ip = openstack_client.get_seed_ip()
        bootstrap = bootstrap_manager.BootstrapManager(seed_ip=seed_ip)

        # step 001
        show_step(1)
        endpoints = openstack_client.get_os_cloud_endpoints()
        bootstrap.step_004_deploy_kaas_regional(region=settings.OS_REGION, endpoints=endpoints)

        # step 002
        show_step(2)
        bootstrap.step_006_wait_for_pods()
        bootstrap.step_006_postdeployment_checks()

        ns = kaas_manager.get_namespace(settings.REGION_NAMESPACE)
        regional_cluster = ns.get_cluster(settings.REGIONAL_CLUSTER_NAME)
        regional_cluster.check.check_machines_status()
        regional_cluster.check.check_cluster_nodes(timeout=900)
        regional_cluster.check.check_helmbundles()
        regional_cluster.check.check_persistent_volumes_mounts()

        regional_cluster.store_k8s_artifacts()

        # step 011
        if settings.OPENSTACK_BOOT_FROM_VOLUME:
            show_step(11)
            bootstrap.check_boot_from_volume(openstack_client,
                                             boot_volume_size=settings.OPENSTACK_BOOT_VOLUME_SIZE)
        # Check/wait for correct docker service replicas in cluster
        regional_cluster.check.check_actual_expected_docker_services()
        regional_cluster.check.check_k8s_pods()
        regional_cluster.check.check_cluster_readiness()
        regional_cluster.check.check_actual_expected_pods(timeout=1200,
                                                          check_all_nss=True)
        regional_cluster.provider_resources.save_artifact()
